from ._compressing_get_ingest_capacity import compressing_get_ingest_capacity


def compressing_has_ingest_capacity(S: int, T: int) -> bool:
    """Does this algorithm have the capacity to ingest a data item at logical
    time T?

    Parameters
    ----------
    S : int
        The number of buffer sites available.
    T : int
        Queried logical time.

    Returns
    -------
    bool

    See Also
    --------
    get_ingest_capacity : How many data item ingestions does this algorithm
    support?
    has_ingest_capacity_batched : Numpy-friendly implementation.

    References
    ----------
    John C. Gunther. 2014. Algorithm 938: Compressing circular buffers. ACM
    Trans. Math. Softw. 40, 2, Article 17 (February 2014), 12 pages.
    https://doi.org/10.1145/2559995
    """
    assert T >= 0
    ingest_capacity = compressing_get_ingest_capacity(S)
    return ingest_capacity is None or T < ingest_capacity


has_ingest_capacity = compressing_has_ingest_capacity  # lazy loader workaround
